# -------------------------------------------------------------
# code developed by Michael Hartmann during his Ph.D.
# Reachability Analysis
#
# (C) 2020 Michael Hartmann, Graz, Austria
# Released under GNU GENERAL PUBLIC LICENSE
# email michael.hartmann@v2c2.at
# -------------------------------------------------------------

from reachab.util.visualizer import *
import numpy as np
import scipy.spatial
from scipy import signal
import logging

"""
    Class for reachability analysis
"""
class reachability(object):
    def __init__(self, **kwargs):
        self.params={'T': kwargs.get('T',2.2),
                     'N': kwargs.get('N',4),
                     'gamma': 0.01 #threshold for control input constraint (inf-norm)
                     }
        self.obj_visual = visualizer()
        self.params['r']=self.params['T']/(self.params['N']+1)
        self.sys={'A': None, 'B': None, 'C': None, 'D': None}
        self.init_fcn()

    """
       Initial function to get system dynamics
    """
    def init_fcn(self):
        A = np.matrix([[0, 0, 1, 0],
                       [0, 0, 0, 1],
                       [0, 0, 0, 0],
                       [0, 0, 0, 0]])
        B = np.matrix([[0, 0],
                       [0, 0],
                       [1, 0],
                       [0, 1]])
        C = np.eye(4)
        D = np.zeros((4, 2))
        self.system_dynamics(A, B, C, D)

    """
        Initial step to start logging
    """
    def start_logging(self):
        logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)

    """
        System dynamics
    """
    def system_dynamics(self, A, B, C, D):
        self.discrete_sys=signal.StateSpace(A, B, C, D, dt=self.params['r'])
        self.sys['A']=np.array(self.discrete_sys.A)
        self.sys['B'] = np.array(self.discrete_sys.B)
        self.sys['C'] = np.array(self.discrete_sys.C)
        self.sys['D'] = np.array(self.discrete_sys.D)

        #see Otto Föllinger, "Regelungstechnik" and Thesis of Matthias Althoff:
        self.Phi=np.eye(4)+self.params['r']*self.sys['A']+1/(2)*(self.sys['A']*self.params['r'])**2+1/(6)*(self.sys['A']*self.params['r'])**3+1/(24)*(self.sys['A']*self.params['r'])**4

    """
        Mulitplication with the center
    """
    def multiplication_on_center(self, mat):
        return mat*np.matrix(self.zonotype['c'])

    """
        Multiplication with a generator list
    """
    def multiplication_on_generator(self, mat, list):
        return mat*np.matrix(list)

    """
        Convert to Matrix
    """
    def convert_to_matrix(self, list):
        erg=np.hstack((list[0], list[1]))
        for i in range(2, len(list)):
            erg=np.hstack((erg, list[i]))
        return erg

    """
        Mulitplication on a zonotype
    """
    def multiplication_on_zonotype(self, mat, zonotype):
        Z = {'c': None, 'g': None}
        Z['c']=mat*zonotype['c']
        for i in range(0, np.size(zonotype['g'], 1)):
            act_g=zonotype['g'][:,i]
            if(i==0):
                g=mat*act_g
            else:
                g=np.hstack((g, mat*act_g))
        Z['g']=g
        return Z

    """
        Get unique vectors
    """
    def get_unique_vectors(self, vec):
        unique_vec = np.squeeze(np.unique(vec, axis=0))
        return unique_vec

    """
        Get edge points of zonotype with convex hull operation
    """
    def get_points_of_zonotype(self, zonotype):
        c=zonotype['c']
        g=zonotype['g']
        x_vec = [c]
        for wlt in range(0, np.size(g,1)):
            a=g[:, wlt]
            x_pos = [i+a for i in x_vec]
            x_neg = [i-a for i in x_vec]
            x_vec=x_pos+x_neg
        unique_vec=self.get_unique_vectors(x_vec)
        try:
            points = self.compute_convex_hull(unique_vec[:, 0], unique_vec[:, 1])
        except:
            None
        return [points[:, 0], points[:, 1]], unique_vec

    """
        Compute the convex hull
    """
    def compute_convex_hull(self, x, y):
        v=np.transpose(np.vstack([x, y]))
        hull = scipy.spatial.ConvexHull(v)
        return hull.points[hull.vertices]

    """
        Minkowski sum with two zonotypes
    """
    def minkowski_zonotypes(self, ZA, ZB):
        Z={'c': None, 'g': None}
        Z['c']=ZA['c']+ZB['c']
        a=ZA['g']
        b=ZB['g']
        new_g=np.hstack((a, b))
        Z['g'] = new_g
        return Z


    """
        Get Box Hull
    """
    def get_box_hull(self, Omega):
        Z = {'c': None, 'g': None}
        r=self.get_points_of_zonotype(Omega)
        std=(((np.max(r[0])-np.min(r[0]))/2, (np.max(r[1])-np.min(r[1]))/2))
        q=np.diag(std)
        Z['c']=Omega['c']
        Z['g'] = np.matrix(np.vstack((q,np.zeros((2,2)))))
        return Z

    """
        Approximation of Reachability analysis:
        based on algorithm 1 from: Girard, A.; "Efficient Computation of Reachable Sets of Linear Time-Invariant Systems 
        with Inputs"
    """
    def approximate_reachable_set_without_box(self, Omega_0, U):
        all_R = []
        all_X = []
        '''
        algorithm 1 from: Girard, A.; "Efficient Computation of Reachable Sets of Linear Time-Invariant Systems with 
        Inputs"
        '''
        all_R.append(Omega_0)
        all_X.append(Omega_0)
        #self.params['N']
        logging.info("number of generators Omega_0: " + str(np.size(Omega_0['g'], 1)))
        logging.info("number of generators U: " + str(np.size(U['g'], 1)))
        # 1. step
        X_0=Omega_0
        X_i=X_0
        # 2. step
        V_0=U
        V_i=V_0
        # 3. step
        S_0={'c': np.matrix([[0],
                             [0],
                             [0],
                             [0],
                                   ]),
                   'g': np.matrix([[0],
                                   [0],
                                   [0],
                                   [0]
                                   ])
                   }
        S_i=S_0
        logging.info("number of generators S_0: " + str(np.size(S_0['g'], 1)))
        # 4. step
        for i in range(0, self.params['N']):
            logging.info("cycle i: "+str(i))
            # 5. step
            X_i = self.multiplication_on_zonotype(self.Phi, X_i)
            all_X.append(X_i)
            logging.info("number of generators X_i: "+str(np.size(X_i['g'],1)))
            # 6. step
            S_i=self.minkowski_zonotypes(S_i, V_i)
            logging.info("number of generators S_i: "+str(np.size(S_i['g'],1)))
            # 7. step
            V_i = self.multiplication_on_zonotype(self.Phi, V_i)
            logging.info("number of generators V_i: "+str(np.size(V_i['g'],1)))
            # 8. step
            Omega_i=self.minkowski_zonotypes(X_i, S_i)
            logging.info("number of generators Omega_i: "+str(np.size(Omega_i['g'],1)))
            all_R.append(Omega_i)
        return all_R, all_X

    """
            Approximation of Reachability analysis:
            based on algorithm 2 from: Girard, A.; "Efficient Computation of Reachable Sets of Linear Time-Invariant 
            Systems with Inputs"
    """
    def approximate_reachable_set_with_box(self, Omega_0, U):
        all_R = []
        all_X = []
        '''
        algorithm 2 from: Girard, A.; "Efficient Computation of Reachable Sets of Linear Time-Invariant Systems with 
        Inputs"
        '''
        all_R.append(Omega_0)
        all_X.append(Omega_0)

        #self.params['N']
        logging.info("number of generators Omega_0: " + str(np.size(Omega_0['g'], 1)))
        logging.info("number of generators U: " + str(np.size(U['g'], 1)))
        # 1. step
        X_0=Omega_0
        X_i=X_0
        # 2. step
        V_0=U
        V_i=V_0
        # 3. step
        S_0={'c': np.matrix([[0],
                             [0],
                             [0],
                             [0],
                                   ]),
                   'g': np.matrix([[0],
                                   [0],
                                   [0],
                                   [0]
                                   ])
                   }
        S_i=S_0
        logging.info("number of generators S_0: " + str(np.size(S_0['g'], 1)))
        # 4. step
        for i in range(0, self.params['N']):
            logging.info("cycle i: "+str(i))
            # 5. step
            X_i = self.multiplication_on_zonotype(self.Phi, X_i)
            all_X.append(X_i)
            logging.info("number of generators X_i: "+str(np.size(X_i['g'],1)))
            # 6. step
            V_i_box = self.get_box_hull(V_i)
            S_i=self.minkowski_zonotypes(S_i, V_i_box)
            logging.info("number of generators S_i: "+str(np.size(S_i['g'],1)))
            # 7. step
            V_i = self.multiplication_on_zonotype(self.Phi, V_i)
            logging.info("number of generators V_i: "+str(np.size(V_i['g'],1)))
            # 8. step
            Omega_i=self.minkowski_zonotypes(X_i, S_i)
            Omega_i=self.get_box_hull(Omega_i)
            logging.info("number of generators Omega_i: "+str(np.size(Omega_i['g'],1)))
            all_R.append(Omega_i)
        return all_R, all_X

    """
        Approximation of Reachability analysis:
        based on algorithm from: Girard, A.; "Reachability of Uncertain Linear Systems
        Using Zonotopes"
    """
    def approximate_reachable_set(self):
        '''
        algorithm from: Girard, A.; "Reachability of Uncertain Linear Systems
        Using Zonotopes"
        '''
        # 1. step
        #see self.params['N']
        inf_norm_A=np.linalg.norm(self.A, np.inf)
        r_norm_A=self.params['r']*inf_norm_A
        exp_r_norm_A=np.exp(r_norm_A)
        exp_r_A = np.exp(self.params['r']*self.A)
        # 2. step
        alpha_r=(exp_r_norm_A-1-r_norm_A)
        # 3. step
        beta_r=(exp_r_norm_A-1)*self.params['gamma']/inf_norm_A
        # 4. step
        P_0={'c': None, 'g': None}
        P_0['c']=np.array((self.zonotype['c']+self.multiplication_on_center(exp_r_norm_A))/2)
        a = [0.5*(i+self.multiplication_on_generator(exp_r_norm_A, i)) for i in self.zonotype['g']]
        b = [0.5 * (i + self.multiplication_on_generator(exp_r_norm_A, i)) for i in self.zonotype['g']]
        P_0['g']=self.get_unique_vectors(a+b)
        # 5. step
        rad=alpha_r+beta_r
        square_Z=self.square_zonotype(rad)
        Q_0=self.minkowski_zonotypes(P_0, square_Z)
        # 6. step
        all_R=[]
        Q_i = Q_0
        R_i=Q_0
        all_R.append(R_i)
        for i in range(1,self.params['N']-1):
            # 7. step
            P_i=self.multiplication_on_zonotype(exp_r_norm_A, Q_i)
            # 8. step
            square_Z = self.square_zonotype(beta_r)
            Q_i = self.minkowski_zonotypes(P_i, square_Z)
            # 9. step
            all_R.append(Q_i)
        return all_R

    """
        Center trajectory
    """
    def center_trajectory(self, R):
        erg=[]
        for i in range(0, len(R)):
            x = np.float(R[i]['c'][0])
            y = np.float(R[i]['c'][1])
            erg.append((x,y))
        return erg

    """
        Test function
    """
    def test_function(self):
        Omega_0 = {'c': np.matrix([[0],
                                   [0],
                                   [10],
                                   [3]
                                   ]),
                   'g': np.matrix([[1, -1, 1, .2, .2],
                                   [1, 1, .3, .2, .5],
                                   [0, 0, 0, .4, .3],
                                   [0, 0, 0, .2, .4]
                                   ])
                   }
        U = {'c': np.matrix([[0],
                             [0],
                             [0],
                             [0],
                             ]),
             'g': np.matrix([[1, 0, 1],
                             [1, 1, 0],
                             [0, 0, 0],
                             [0, 0, 0]
                             ])
             }
        program = ['without_box', 'with_box']
        program_select = 0
        if (program[0] == program[program_select]):
            R, X = self.approximate_reachable_set_without_box(Omega_0, U)
        elif (program[1] == program[program_select]):
            R, X = self.approximate_reachable_set_with_box(Omega_0, U)
        for act_zono in R:
            zonoset_P0 = self.get_points_of_zonotype(act_zono)
            self.obj_visual.filled_polygon(zonoset_P0, 'green', .2)
        for act_zono in X:
            zonoset_P0 = self.get_points_of_zonotype(act_zono)
            self.obj_visual.filled_polygon(zonoset_P0, 'orange')
        traj = self.center_trajectory(R)
        self.obj_visual.show_traj(traj)
        self.obj_visual.show()

if __name__ == '__main__':
    obj_reach = reachability(**{'T':3, 'N':5})
    obj_reach.start_logging()
    obj_reach.test_function()



# -------------------------------------------------------------
# code developed by Michael Hartmann during his Ph.D.
# Reachability Analysis
#
# (C) 2020 Michael Hartmann, Graz, Austria
# Released under GNU GENERAL PUBLIC LICENSE
# email michael.hartmann@v2c2.at
# -------------------------------------------------------------
import reachab.src.reachability as rb
import matplotlib.pyplot as plt
from scipy.signal import savgol_filter
from shapely.geometry import Polygon, Point
import numpy as np
from reachab.src.states_reachsets import *



def points_inside_hull(zonoset):
    all_inside_points=get_sample_points_inside_hull(zonoset)
    return all_inside_points


def plot_all_inside_points(all_inside_points):
    colors=["red", "orange", "yellow", "cyan", "blue", "black"]
    for idx, act_points in enumerate(all_inside_points):
        for act_point in act_points:
            if(idx<=len(colors)):
                plt.arrow(act_point[0], act_point[1], 0.1*act_point[2], 0.1*act_point[3],
                      fc=colors[idx], ec="black", alpha=.3, width=.1,
                      head_width=.4, head_length=.63)
            else:
                plt.arrow(act_point[0], act_point[1], 0.1 * act_point[2], 0.1 * act_point[3],
                          fc="red", ec="black", alpha=.3, width=.1,
                          head_width=.4, head_length=.63)




import logging
'''
    Test me function. A user should see if the program is installed and a simple plot is available 
'''
def test_me():
    obj_reachability = rb.reachability()
    obj_reachability.test_function()



'''
    Reachability Analysis. 
    Omega_0: Initial state set
    U: Fixed control input set
    Params: Parameters
'''
def reach(Omega_0, U, params):
    ##################################
    ## REACHABILITY ANALYSIS PARAMS ##
    ##################################
    ra_params = {
        'T': params['time_horizon'],
        'N': params['steps'],
    }
    erg=[]
    obj_reach = rb.reachability(**ra_params)
    program = ['without_box', 'with_box']
    if (program[0] == params['box_function']):
        R, X = obj_reach.approximate_reachable_set_without_box(Omega_0, U)
    elif (program[1] == params['box_function']):
        R, X = obj_reach.approximate_reachable_set_with_box(Omega_0, U)
    for act_zono in R:
        zonoset, unique_vec = obj_reach.get_points_of_zonotype(act_zono)
        if (params['visualization'] == 'y'):
            obj_reach.obj_visual.filled_polygon(zonoset, params['face_color'], .2)
        erg.append(zonoset)
    #points_a=states_for_R(params, obj_reach, R)
    return erg

def reach_zonotype_without_box(Omega_0, U, **kwargs):
    ##################################
    ## REACHABILITY ANALYSIS PARAMS ##
    ##################################
    erg_zonoset=[]
    ra_params = {
        'T': kwargs["time_horizon"],
        'N': kwargs["steps"],
    }
    obj_reach = rb.reachability(**ra_params)
    R, X = obj_reach.approximate_reachable_set_without_box(Omega_0, U)
    for act_zono in R:
        zonoset, unique_vec = obj_reach.get_points_of_zonotype(act_zono)
        if (kwargs['visualization'] == 'y'):
            obj_reach.obj_visual.filled_polygon(zonoset, kwargs['face_color'], .2)
        erg_zonoset.append(unique_vec)
    return R, X, obj_reach, erg_zonoset

def get_zonotype_points(obj_reach, R):
    erg = []
    for act_zono in R:
        zonoset = obj_reach.get_points_of_zonotype(act_zono)
        erg.append(zonoset)
    return erg

'''
    Show the plots
'''
def show_all():
    plt.axis('equal')
    plt.grid("on")
    plt.show()


'''
    Get states for reach sets R
'''
def states_for_R(params, obj_reach, R):
    act_zono=R[-1]
    points=obj_reach.get_points_of_zonotype(act_zono)
    tuple_points=[(points[0][i], points[1][i]) for i in range(0, len(points[0]))]
    poly = Polygon(tuple_points)
    minx, miny, maxx, maxy = poly.bounds
    x=np.linspace(minx, maxx, params['nx'])
    y=np.linspace(miny, maxy, params['ny'])
    xv, yv = np.meshgrid(x, y)
    x_vec=np.ravel(xv)
    y_vec = np.ravel(yv)

    list_points=[]
    for i in range(0, len(x_vec)):
        a_x=x_vec[i]
        a_y=y_vec[i]
        # print(a_x, a_y)
        new_point=Point(a_x, a_y)
        bool_test=is_point_in_polygon(new_point, poly)
        if(bool_test == True):
            list_points.append((a_x, a_y))
    return list_points


'''
    Is point in polygon
'''
def is_point_in_polygon(point, poly):
    if poly.contains(point):
        return True
    else:
        return False

'''
    Get states for reach sets R
'''
def get_points_zonotype(zonotype):
    obj_reachability = rb.reachability()
    return obj_reachability.get_points_of_zonotype(zonotype)

'''
    Compute velocity with Savgol Golay Filter
'''
def compute_velocity(trajectory, window_size=70, polyorder=2):
    rx = trajectory[0]
    ry = trajectory[1]
    vx = savgol_filter(rx, window_size, polyorder, 1)
    vy = savgol_filter(ry, window_size, polyorder, 1)
    return vx,vy

'''
    compute next position
'''
def compute_next_position(vx, vy, x, y, Ts):
    x = Ts*vx+x
    y = Ts*vy+y
    return x, y

'''
    compute next velocity
'''
def compute_next_velocity(vx, vy, ax, ay, Ts):
    vx = Ts*ax+vx
    vy = Ts*ay+vy
    return vx, vy


'''
    compute cumulative distance of trajectory
'''
def compute_cumulative_distance(x, y):
    dif_x = [(x[i+1]-x[i])**2 for i in range(0, len(x)-1)]
    dif_y = [(y[i+1]-y[i])**2 for i in range(0, len(y)-1)]
    distances=[np.sqrt(dif_x[i]+dif_y[i]) for i in range(0, len(dif_x))]
    total_distance=np.sum(distances)
    return total_distance




# -------------------------------------------------------------
# code developed by Michael Hartmann during his Ph.D.
# Reachability Analysis
#
# (C) 2020 Michael Hartmann, Graz, Austria
# Released under GNU GENERAL PUBLIC LICENSE
# email michael.hartmann@v2c2.at
# -------------------------------------------------------------

import setuptools
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='reachab',
      version='0.4.10',
      description='Reachability Analysis with zonotypes',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/ga74kud/reachab',
      author='Michael Hartmann',
      author_email='michael.hartmann@v2c2.at',
      license='GNU GENERAL PUBLIC LICENSE',
      packages=setuptools.find_packages(),
      install_requires=[
          "scipy",
          "numpy",
          "matplotlib",
          "argparse",
          "logging",
        ],
      zip_safe=False)



import matplotlib.pyplot as plt
import numpy as np

class visualizer(object):
    def __init__(self, **kwargs):
        None
    def set_new_figure(self):
        plt.figure()

    def line_between(self, a, b, col):
        plt.plot([a[0], b[0]], [a[1], b[1]], color=col, alpha=.5)
    def show(self):
        plt.axis('equal')
        plt.grid("on")
        plt.show()

    def get_colormap(self, colmap):
        cmap = plt.get_cmap(colmap).colors
        return np.array(cmap)
    def show_point(self, point):
        plt.plot((point[0]), (point[1]), 'x', color='r')

    def filled_polygon(self, zonoset, face_color, alpha=.1):
        x=zonoset[0]
        y=zonoset[1]
        plt.fill(x, y, facecolor=face_color, edgecolor='k', linewidth=3, alpha=alpha)

    def show_traj(self, points):
        cmap = self.get_colormap("plasma")
        col_idx = np.floor(np.linspace(0, np.size(cmap, 0) - 1, np.size(points, 0)))
        col_idx_int = [int(wlt) for wlt in col_idx]

        for idx in range(0, len(points)-1):
            act_point=points[idx]
            next_point = points[idx+1]
            x_pos = act_point[0]
            y_pos = act_point[1]
            x_dif=next_point[0]-x_pos
            y_dif=next_point[1]-y_pos
            scale = 1
            act_col=cmap[col_idx_int[idx], :]
            plt.arrow(x_pos, y_pos, scale * x_dif, scale * y_dif,
                          fc=act_col, ec="black", alpha=.65, width=.4,
                          head_width=1.4, head_length=1)
        plt.axis([-11.5, 11.5, -11.5, 11.5])
        plt.legend()
        plt.xlabel('x')
        plt.ylabel('y')



# -------------------------------------------------------------
# code developed by Michael Hartmann during his Ph.D.
# Reachability Analysis
#
# (C) 2021 Michael Hartmann, Graz, Austria
# Released under GNU GENERAL PUBLIC LICENSE
# email michael.hartmann@v2c2.at
# -------------------------------------------------------------

import matplotlib.pyplot as plt
import numpy as np
from scipy.optimize import linprog

def show_points(points, color):
    plt.scatter(points[:, 0], points[:, 1], edgecolors=color)


def show_filled_polygon(points, the_color):
    plt.fill(points[:, 0], points[:, 1], color=the_color, alpha=.2)
'''
    is x inside the points 
    idea is to solve a linear program
    original code from "https://stackoverflow.com/questions/16750618/whats-an-efficient-way-to-find-if-a-point-lies-in-the-convex-hull-of-a-point-cl"
'''
def inside_polygon(points, x):
    n_points = len(points)
    c = np.zeros(n_points)
    A = np.r_[points.T,np.ones((1,n_points))]
    b = np.r_[x, np.ones(1)]
    lp = linprog(c, A_eq=A, b_eq=b)
    return lp.success

def inside_polygon_ND(points, x):
    n_points = np.size(points, 0)
    c = np.zeros(n_points)
    A = np.r_[points.T,np.ones((1,n_points))]
    b = np.r_[x, np.ones(1)]
    lp = linprog(c, A_eq=A, b_eq=b)
    return lp.success

def check_points_inside(points, polygon_points):
    inside_points=[]
    for wlt in range(0, np.size(points, 0)):
        act_point=points[wlt, :]
        bool_val=inside_polygon(polygon_points, act_point)
        if(bool_val):
            inside_points.append(act_point)
    inside_points=np.array(inside_points)
    return inside_points
def check_points_inside_ND(points, polygon_points):
    inside_points=[]
    for wlt in range(0, np.size(points, 0)):
        act_point=points[wlt, :]
        bool_val=inside_polygon_ND(polygon_points, act_point)
        if(bool_val):
            inside_points.append(act_point)
    inside_points=np.array(inside_points)
    return inside_points
def get_points_inside_ND(act_points):
    all_extrema=[(np.min(act_points[:, i]), np.max(act_points[:, i])) for i in range(0, np.size(act_points, 1))]
    all_linspaces=[np.linspace(i[0], i[1], 5) for i in all_extrema]
    X, Y, VX, VY = np.meshgrid(*all_linspaces)
    X, Y, VX, VY = np.ravel(X), np.ravel(Y), np.ravel(VX), np.ravel(VY)
    meshgrid_points = np.transpose(np.vstack(([X, Y, VX, VY])))
    meshgrid_points= np.unique(meshgrid_points, axis=0)
    inside_points = check_points_inside_ND(meshgrid_points, act_points)
    return inside_points
def get_points_inside(polygon_points):
    x = polygon_points[:, 0]
    y = polygon_points[:, 1]
    xmin,xmax,ymin,ymax=np.min(x), np.max(x), np.min(y), np.max(y)
    x_lin, ylin=np.linspace(xmin, xmax, 10), np.linspace(ymin, ymax, 10)
    X, Y=np.meshgrid(x_lin, ylin)
    X,Y=np.ravel(X), np.ravel(Y)
    meshgrid_points=np.transpose(np.vstack((X, Y)))
    inside_points=check_points_inside(meshgrid_points, polygon_points)
    return inside_points
def get_sample_points_inside_hull(zonoset):
    all_inside_points=[]
    for act_points in zonoset:
        points=np.transpose(np.vstack((act_points[:, 0], act_points[:, 1])))
        #inside_points=get_points_inside(points)
        inside_points = get_points_inside_ND(act_points)
        all_inside_points.append(inside_points)
    return all_inside_points


if __name__ == '__main__':
    n_points = 10000
    n_dim = 2
    Z = np.array([[0, 5], [4, 2], [8, 4], [8, 5], [6, 8], [1, 6]])
    x = np.array([[4, 5]])



    for act_point in x:
        print(inside_polygon(Z, act_point))

    show_filled_polygon(Z, 'green')
    show_points(Z, 'blue')
    show_points(x, 'red')
    plt.show()